<%@ Application Language="C#" ClassName="Global" %>
<%@ Import Namespace="System.Data" %>
<%@ Import Namespace="System.Data.SqlClient" %>
<%@ Import Namespace="System.Security.Cryptography" %>
<%@ Import Namespace="CuteChat" %>

<script RunAt="server">


	public override void Init()
	{
		base.Init();

		lock(typeof(CuteChat.ChatSystem))
		{
			if(!CuteChat.ChatSystem.HasStarted)
			{
                CuteChat.ChatProvider.Instance = new SnitzChatProvider();
				CuteChat.ChatSystem.Start(new CuteChat.AppSystem());
			}
		}
	}


    static string _connstr;
    static public string ConnectionString
    {
        get
        {
            if (_connstr == null)
            {
                string connstr = System.Configuration.ConfigurationSettings.AppSettings["ConnectionString"];
                if (connstr == null || connstr == "")
                {
                    throw (new Exception("Missing web.config appSettings : ConnectionString"));
                }
                _connstr = connstr;
            }
            return _connstr;
        }
    }

   static public SqlConnection CreateConnection()
    {
        return new SqlConnection(ConnectionString);
    }
    static private void FillParameters(SqlCommand cmd, object[] parameters)
    {
        if (parameters == null) return;
        for (int i = 0; i < parameters.Length; i++)
        {
            SqlParameter param;
            object pvalue = parameters[i];
            if (pvalue == null || Convert.IsDBNull(pvalue))
            {
                param = new SqlParameter("@p" + i, SqlDbType.NVarChar, 50);
                param.Value = DBNull.Value;
            }
            else if (pvalue is string)
            {
                param = new SqlParameter("@p" + i, SqlDbType.NVarChar, pvalue.ToString().Length + 100);
                param.Value = pvalue;
            }
            else if (pvalue is SqlParameter)
            {
                param = (SqlParameter)pvalue;
            }
            else
            {
                param = new SqlParameter("@p" + i, (object)pvalue);
            }
            cmd.Parameters.Add(param);
        }
    }
    static public SqlCommand CreateCommand(string cmdtext, params object[] parameters)
    {
        SqlCommand cmd = new SqlCommand(cmdtext, CreateConnection());
        FillParameters(cmd, parameters);
        return cmd;
    }
    static public SqlDataAdapter CreateAdapter(string cmdtext, params object[] parameters)
    {
        SqlCommand cmd = new SqlCommand(cmdtext, CreateConnection());
        FillParameters(cmd, parameters);
        return new SqlDataAdapter(cmd);
    }
    static public SqlDataReader ExecuteReader(string cmdtext, params object[] parameters)
    {
        SqlCommand cmd = new SqlCommand(cmdtext, CreateConnection());
        FillParameters(cmd, parameters);
        cmd.Connection.Open();
        return cmd.ExecuteReader(CommandBehavior.CloseConnection);
    }
    static public object ExecuteScaler(string cmdtext, params object[] parameters)
    {
        SqlCommand cmd = new SqlCommand(cmdtext, CreateConnection());
        FillParameters(cmd, parameters);
        cmd.Connection.Open();
        using (SqlDataReader sdr = cmd.ExecuteReader(CommandBehavior.CloseConnection))
        {
            if (!sdr.Read())
                return null;

            if (sdr.IsDBNull(0))
                return DBNull.Value;

            return sdr.GetValue(0);
        }
    }


    static string _prefix;
    static public string SnitzPrefix
    {
        get
        {
            if (_prefix == null)
            {
                string pre = System.Configuration.ConfigurationSettings.AppSettings["SnitzPrefix"];
                if (pre == null || pre == "")
                {
                    throw (new Exception("Missing web.config appSettings : SnitzPrefix"));
                }
                _prefix = pre;
            }
            return _prefix;
        }
    }

    static string _snitzid;
    static public string SnitzID
    {
        get
        {
            if (_snitzid == null)
            {
                string id = System.Configuration.ConfigurationSettings.AppSettings["SnitzUniqueID"];
                if (id == null || id == "")
                {
                    throw (new Exception("Missing web.config appSettings : SnitzUniqueID"));
                }
                _snitzid = id;
            }
            return _snitzid;
        }
    }

    static public string SHA256(string str)
    {
        byte[] data = ConvertToWordArray(str);
        byte[] hash;

        SHA256Managed sha = new SHA256Managed();
        try
        {
            hash = sha.ComputeHash(data);
        }
        finally
        {
            sha.Clear();
        }
        return BitConverter.ToString(hash).Replace("-", "");
    }

    //Snitz/inc_sha256
    static private byte[] ConvertToWordArray(string str)
    {
        //OK,,Only support ascii code , for the password only..
        return System.Text.Encoding.ASCII.GetBytes(str);
    }

    static public bool CheckUser(string username, string pwdencry)
    {
        if (username == null || username == "") return false;
        if (pwdencry == null || pwdencry == "") return false;

        using (SqlDataReader sdr = ExecuteReader("SELECT M_PASSWORD FROM " + SnitzPrefix + "MEMBERS WHERE M_NAME=@p0 AND M_STATUS=1"
                    , username))
        {
            if (!sdr.Read())
                return false;

            string pwd = sdr.GetString(0);

            if (pwd == null || pwd == "")
                return false;

            return pwd.ToUpper() == pwdencry.ToUpper();
        }
    }
        

    public class SnitzChatProvider : ChatProvider
    {
        public override string GetConnectionString()
        {
            return ConnectionString;
        }



        public override AppChatIdentity GetLogonIdentity()
        {
            HttpContext context = HttpContext.Current;
            object obj = context.Items["AppChatIdentity"];
            if (obj != null)
                return obj as AppChatIdentity;

            AppChatIdentity identity = null;
            HttpCookie cookie = context.Request.Cookies[SnitzID + "User"];
            if (cookie != null)
            {
                string username = cookie["Name"];
                string pwdencry = cookie["Pword"];

                if (CheckUser(username, pwdencry))
                {
                    identity = new AppChatIdentity(username, false, ToUserId(username), context.Request.UserHostAddress);
                }
            }

            obj = DBNull.Value;
            if (identity != null)
            {
                obj = identity;
            }

            context.Items["AppChatIdentity"] = obj;

            return obj as AppChatIdentity;
        }

        public override string FindUserLoginName(string nickName)
        {
            if (GetUserInfo(nickName, ref nickName))
                return nickName;
            return null;
        }
        public override bool GetUserInfo(string loginName, ref string nickName, ref bool isAdmin)
        {
            using (SqlDataReader sdr = ExecuteReader("SELECT M_NAME,M_LEVEL FROM " + SnitzPrefix + "MEMBERS WHERE M_NAME = @p0"
                        , loginName))
            {
                if (sdr.Read())
                {
                    nickName = sdr.GetString(0);
                    //M_LEVEL>=3
                    isAdmin=Convert.ToInt32(sdr.GetValue(1)) >= 3;
                    return true;
                }
            }
            return false;
        }
        public override bool ValidateUser(string username, string password)
        {
            if (username == null || username == "") return false;
            if (password == null || password == "") return false;

            password = SHA256(password);

            if (!CheckUser(username, password))
                return false;

            HttpCookie cookie = HttpContext.Current.Response.Cookies[SnitzID + "User"];
            cookie["Name"] = username;
            cookie["Pword"] = password;//encrypted..

            return true;
        }
    }



</script>

